package main

import (
	"context"
	"fmt"
	"time"

	influxdb2 "github.com/influxdata/influxdb-client-go/v2"
)

type InfluxDB struct {
	client influxdb2.Client
}

func ConnectInfluxDB(remote string, token string) (InfluxDB, error) {
	var influx InfluxDB
	influx.client = influxdb2.NewClient(remote, token)
	return influx, nil
}

func (influx *InfluxDB) Ping(timeout time.Duration) (time.Duration, error) {
	ctx, cancel := context.WithTimeout(context.Background(), timeout)
	defer cancel()
	now := time.Now()
	select {
	case <-ctx.Done():
		// timeout
		return time.Duration(0), fmt.Errorf("timeout")
	default:
		succ, err := influx.client.Ping(ctx)
		delta := time.Now().Sub(now)
		if err != nil {
			return delta, err
		}
		if !succ {
			return delta, fmt.Errorf("no reply from server")
		}
		return delta, nil
	}
}

// Write a measurement into the database
func (influx *InfluxDB) Write(org, bucket, measurement string, tags map[string]string, fields map[string]interface{}) error {
	writeAPI := influx.client.WriteAPIBlocking(org, bucket)
	p := influxdb2.NewPoint(measurement, tags, fields, time.Now())
	err := writeAPI.WritePoint(context.Background(), p)
	return err
}
